#################################################################################
# Created       : Martin Foster
# Modified      : 05/29/2001
#################################################################################
#
# Post - Module part of Webchat designed to display posts based on information
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Webchat::Post;						# Package name

use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter functionality


#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# GetPosts
#
# Using basic information about the user will return a Statement handle (more 
# lightweight then an array) that will be used in the display the information
# gathered.

sub GetPosts							# Database, Statement reference, Parameters
{
	#####################
	# Data members
	my $Database  = shift;					# Database handle
	my $Statement = shift;					# Statement handle (reference)

	my $param     = shift;					# Parameter list
	my $realm     = shift;					# Reference to realm hash

	my $formatting;						# Formatting used	
	my $delay;						# Delay to retreive posts

	my $ignored;						# Ignored person
		

	#####################
	# Program area

	# Determine action 
	if ((!defined($param->{'NUM'})) || (defined($param->{'SCROLLBACK'})))
	{ 
		# Time of delay
		$delay = (time - $realm->{'SetTimeout'}); 

		# Cleanup
		delete $param->{'SCROLLBACK'};
	}	


	# Format handling
	# Retreive formatting used
	($formatting) = Webchat::Database::DataGetFormatting($Database, $param->{'USER'});

	# Determin field to use
	if ($formatting eq 'complete')				# Full formatting
	{ $formatting = 'PostFullFormat'; }
	elsif ($formatting eq 'spartan')			# Spartan formatting
	{ $formatting = 'PostPartialFormat'; }
	elsif ($formatting eq 'imageless')			# Full less images
	{ $formatting = 'PostImagelessFormat'; }

	# List of ignored
	$ignored = IgnoreHandler($Database, $param);

	# Number based search
	unless (defined($delay))
	{
		# Initialize and execute
		${ $Statement } = $Database->prepare("SELECT PostIDNumber, $formatting FROM Post 
					WHERE PostIDNumber > ? AND RealmName=? AND (PostTo='all' OR PostTo=?) 
					$ignored ORDER BY PostIDNumber");
		${ $Statement }->execute($param->{'NUM'}, $param->{'ROOM'}, $param->{'USER'});
	}
	# Time based search
	else
	{
		# Initialize and execute
		${ $Statement } = $Database->prepare("SELECT PostIDNumber, $formatting FROM Post 
					WHERE PostTimeStamp > ? AND RealmName=? AND (PostTo='all' OR PostTo=?) 
					$ignored ORDER BY PostIDNumber");
		${ $Statement }->execute($delay, $param->{'ROOM'}, $param->{'USER'});
	}
}


#####################
# Handle Posts
#
# Will display all retreived posts to screen, a fairly straightforward 
# operation, will also give a value to NUM once it knows the last post
# received

sub HandlePosts							# Database, Parameters
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Statement handle (reference)
	my $param    = shift;					# Parameter list

	my $post;						# Post itself
	my $postnum;						# Post number

	my %realm;						# Realm hash


	#####################
	# Program area

	# Link with hash
	Webchat::Database::GetHashRealm($Database, \%realm);

	# Retreive posts
	GetPosts($Database, \$Statement, $param, \%realm);	

	# Run though posts and print
	while ( ($_, $post) = $Statement->fetchrow() )
	{ 
		print "$realm{'TagSeperator'}";			# Seperator line
		print $post;					# Display post 
		$postnum = $_;					# Give value to postnum
	}

	# Finish query
	$Statement->finish();
	
	# Change value of NUM or define
	if (defined($postnum))
	{ $param->{'NUM'} = ($postnum - 1); }
}


#####################
# Handle Stream
#
# Will display all retreived posts to screen, a fairly straightforward 
# operation.  This unlike HandlePosts will not stop until the stream
# is ended

sub HandleStream
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Statement handle (reference)

	my $cgi      = shift;					# CGI Handle
	my $param    = shift;					# Parameter list

	my $post;						# Post itself
	my $postnum;						# Post number
	my $postold = 0;					# Old post number
	my $tmp;						# Temporary scalar holder

	my $size    = 0;					# Size of posts
	my $active  = 0;					# Activity counter
	my $elapse  = 0;					# Time elapsed		

	my %realm;						# Chat hash


	#####################
	# Program area

	# Assign seperator
	Webchat::Database::GetHashRealm($Database, \%realm);
	my $seperator = $realm{'TagSeperator'};

	while (($size  < $realm{'SetStreamSize'}) 
	   && ($active < $realm{'SetActive'})
	   && ($elapse < $realm{'SetElapse'}))
	{
		# Retreive posts
		GetPosts($Database, \$Statement, $param, \%realm);	

		# Run though posts and print
		while ( ($tmp, $post) = $Statement->fetchrow() )
		{ 
			# Display
			print "$seperator";			# Seperator line
			print $post;				# Display post 
	
			# Assignment
			$postnum = $tmp;			# Give value to postnum
			$size += length($post);
		}

		# Finish query
		$Statement->finish();
	
		# Change value of NUM or define
		if (defined($postnum))
		{ 
			# Increment 
			$active = ($postold != $postnum ) ? 0 : ($active + 1); 

			# Assign PostOld and Parameter
			$postold 	= $postnum;
			$param->{'NUM'} = $postnum;
		}

		# Print comment
		print $cgi->comment("$postnum"), "\n";

		# Final size calculations
		$size += (length($postnum) + 8);
		
		# Increment Elapsed time
		$elapse++;

		# Sleep before next pass
		sleep(5);
	}

	# To retrieve last made post
	$param->{'NUM'}--;

	# Retreive URL
	my $url = $cgi->url();
	my $inline = Webchat::Param::EmbedInline($Database, $cgi, %{$param});

	# Reload link upon reaching bottom
	print "<P><CENTER><A HREF=\"$url$inline\&MAIN=aff\">$realm{'TagLnkReload'}</A></CENTER></P>";
}


#####################
# Ignore Handler
#
# Will create a string to append to the end of the SQL statement, as
# to ignore the specified list of people.

sub IgnoreHandler						# Database, Parameters
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Statement handle (reference)
	my $param = shift;					# Parameter list

	my $string;						# Ignored string
	my $ignored;						# Ignored person
	my @ignored;						# Ignored group

	#####################
	# Program area

	# Retreive list of the damned
	Webchat::Database::GetListPuppetIgnored($Database, \@ignored, $param->{'USER'});

	# As long as there are entries
	while ($ignored = shift @ignored)
	{
		# Add double quotes if necessary
		$ignored =~ s/'/''/gs;

		# If the string is defined
		if (defined($string))
		{ $string = "$string AND PuppetName <> '$ignored'"; }

		# Initialize for next pass
		else
		{ $string = " AND PuppetName <> '$ignored'"; }		
	}

	# Checks before returning
	if (defined($string))
	{ return $string;  }					# Return string

	else							# Empty string
	{ return ' '; } 
}

1;								# Return true
