#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 07/23/2001
#################################################################################
#
# Realm Configuration - Script part of Webchat designed for access control lists
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Webchat::Audit;						# Audit trail
use Webchat::Database;						# Database handler
use Webchat::Date;						# Date handler
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table handle


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my @allow;							# Allow list
my @block;							# Block list

my %config;							# System hash
my %param;							# Parameter hash


#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle

	# Multiple handle handling
	@allow = $cgi->param('ALLOW');
	@block = $cgi->param('BLOCK');


	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);        # HTML header

	# Login
	if (Webchat::Login::GetVerificationAdmin($Database, $cgi, \%param))
	{
		# Title
		print $cgi->h1('Realm Access Administration'), "\n";

		# Logic Tree
		# Access Control
		if (defined($param{'ROOM'}))
		{ AccessControl($Database, $cgi, \%param, \@allow, \@block); }

		# Realm Listing
		else
		{ RealmSelect($Database, $cgi, \%param); }

	}

	Webchat::Database::DocumentGetFooter($Database);	# HTML Footer
	$Database->disconnect();				# Disconnect from database


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Realm Select
#
# Basic display and prompt user for a selection on which realm to adminster and
# make modifications of some type to.

sub RealmSelect							# Database, CGI, Parameter, config
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash

	my @realms;						# Array of realms
	my $realms;						# Popup menu

	
	#####################
	# Program area

	# Prepare of drop down
	Webchat::Database::GetListNonPrivate($Database, \@realms);
	$realms = $cgi->popup_menu('ROOM', \@realms);

	# HTML Display
	# Star form and top
	print $cgi->start_form();
	Webchat::Table::MakeTop($cgi);

	# Realm selection
	Webchat::Table::MakeValid($cgi, '<B>Select Realm</B>', $realms);
	Webchat::Table::MakeBlank($cgi);

	# Submit and bottom
	Webchat::Table::MakeSingle($cgi, $cgi->submit('Continue'));
	Webchat::Table::MakeBottom($cgi);

	# End of form and embed
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param});
	print $cgi->end_form(); 
}


#####################
# Access Control
#
# Access Control Lists is essentially the role of this script.  This specific 
# subroutine is aimed speficially at handling who can access which Realm
# and perform supervisory functions.

sub AccessControl
{
	#####################
	# Data Members

	my $Database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list

	my $allow    = shift;					# Access granted list
	my $block    = shift;					# Access denied list

	my $handle;						# Handle name

	my $badd     = '>>';					# Addition button
	my $brem     = '<<';					# Remove button

	my $inline;						# Inline parameter list
	my $url      = $cgi->url();				# Self referencing link

	my @allow;						# Allowed
	my @block;						# Blocked


	#####################
	# Program Files

	print $cgi->h2($param->{'ROOM'});

	# Data handling 
	# Additions
	if (defined($param->{$badd}))
	{
		foreach $handle (@{$block})
		{
			# Insert Entry
			$Database->do("INSERT INTO RealmSupervisor 
				(RealmName,
				 PuppeteerLogin)
				VALUES (?,?)", {},

			 $param->{'ROOM'},
			 $handle);

			# Audit table
			Webchat::Audit::AccessAdd($Database, $cgi, $param, $param->{'ROOM'}, $handle);	
		}

		# Cleanup	
		delete $param->{$badd};
	}

	# Removal
	elsif (defined($param->{$brem}))
	{
		foreach $handle (@{$allow})
		{
			# Remove data
			$Database->do("DELETE FROM RealmSupervisor
				WHERE PuppeteerLogin=?
				  AND RealmName=?", {},

				$handle,
				$param->{'ROOM'});

			# Audit table
			Webchat::Audit::AccessAdd($Database, $cgi, $param, $param->{'ROOM'}, $handle);	
		}

		# Cleanup
		delete $param->{$brem};
	}
	


	# Data retreival
	# Retrieve variables
	Webchat::Database::GetList($Database, \@block, "SELECT PuppeteerLogin FROM Puppeteer WHERE PuppeteerSuperPrivs='yes' ORDER BY PuppeteerLogin"); 
	Webchat::Database::GetList($Database, \@allow, "SELECT PuppeteerLogin FROM RealmSupervisor WHERE RealmName=? ORDER BY PuppeteerLogin", $param->{'ROOM'});

	# Generate inline
	($inline) = Webchat::Param::EmbedInline($Database, $cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}); 


	# Form display
	# Form start
	print $cgi->start_form();

	# Display
	print "<CENTER><TABLE WIDTH=\"500\">\n",
	      "\t<TR>\n\t\t<TD COLSPAN=\"3\"><CENTER><B><A HREF=\"$url$inline\">[Back]</A></B></CENTER></TD>\n\t</TR>\n",
	      "\t<TR>\n",
	      "\t\t<TD><B>Supervisors</B></TD>",
	      "\t\t<TD WIDTH=\"50\">&nbsp;</TD>",
	      "\t\t<TD><B>Realm Supervisors</B></TD>",
	      "\t</TR>\n",
	      "\t<TR>\n",
	      "\t\t<TD>", $cgi->scrolling_list('BLOCK', \@block, $block[0], '5', 'true'), "</TD>",
	      "\t\t<TD WIDTH=\"50\">", $cgi->submit($badd), "<BR><BR>", $cgi->submit($brem), "</TD>",
	      "\t\t<TD>", $cgi->scrolling_list('ALLOW', \@allow, $allow[0], '5', 'true'), "</TD>",
	      "\t</TR>\n",
	      "</TABLE></CENTER>";

	# Form end
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param});
	print $cgi->end_form();
}
